@@ -4,6 +4,7 @@
 #' and therefore by definition already resolved upon creation.
 #'
 #' @inheritParams Future-class
+#' 
 #' @param \dots Not used.
 #'
 #' @return An object of class \code{ConstantFuture}.
@@ -11,9 +12,9 @@
 #' @export
 #' @name ConstantFuture-class
 #' @keywords internal
-ConstantFuture <- function(expr=NULL, envir=emptyenv(), substitute=FALSE, local=FALSE, ...) {
+ConstantFuture <- function(expr=NULL, envir=emptyenv(), substitute=FALSE, globals=NULL, packages=NULL, local=FALSE, ...) {
   expr <- force(expr)
-  f <- Future(expr=expr, envir=emptyenv(), substitute=FALSE, local=FALSE, ...)
+  f <- Future(expr=expr, envir=emptyenv(), substitute=FALSE, globals=NULL, packages=NULL, local=FALSE, ...)
   f$value <- expr
   f$state <- "finished"
   structure(f, class=c("ConstantFuture", class(f)))
