@@ -112,8 +112,7 @@ availableCores <- function(constraints=NULL, methods=getOption("future.available
       ## Number of cores assigned by Sun/Oracle Grid Engine (SGE)
       n <- getenv("NSLOTS")
     } else if (method == "mc.cores") {
-      n <- getopt("mc.cores") + 1L
-      .Deprecated(msg="Method 'mc.cores' for future::availableCores() is deprecated; use 'mc.cores+1' instead.")
+      .Defunct(msg = "Method 'mc.cores' for future::availableCores() is deprecated; use 'mc.cores+1' instead.")
     } else if (method == "mc.cores+1") {
       ## Number of cores by option defined by 'parallel' package
       n <- getopt("mc.cores") + 1L
