@@ -18,8 +18,8 @@
 
   ## Temporarily use a different plan
   oplan <- plan("list")
-  on.exit(plan(oplan, substitute=FALSE, .call=NULL))
-  plan(strategy, substitute=FALSE, .call=NULL)
+  on.exit(plan(oplan, substitute=FALSE, .call=NULL, .cleanup=FALSE, .init=FALSE, .check_lazy=FALSE))
+  plan(strategy, substitute=FALSE, .call=NULL, .cleanup=FALSE, .init=FALSE, .check_lazy=FALSE)
 
   eval(fassignment, envir=envir)
 }
