@@ -1,4 +1,4 @@
-#' Create a remote future whose value will be resolved asynchroneously in a remote process
+#' Create a remote future whose value will be resolved asynchronously in a remote process
 #'
 #' A remote future is a future that uses remote cluster evaluation,
 #' which means that its \emph{value is computed and resolved
@@ -18,7 +18,7 @@
 #' Note that remote futures use \code{persistent=TRUE} by default.
 #'
 #' @export
-remote <- function(expr, envir=parent.frame(), substitute=TRUE, globals=TRUE, persistent=TRUE, workers=NULL, user=NULL, revtunnel=TRUE, gc=FALSE, earlySignal=FALSE, myip=NULL, label=NULL, ...) {
+remote <- function(expr, envir=parent.frame(), substitute=TRUE, lazy=FALSE, seed=NULL, globals=TRUE, persistent=TRUE, workers=NULL, user=NULL, revtunnel=TRUE, gc=FALSE, earlySignal=FALSE, myip=NULL, label=NULL, ...) {
   if (substitute) expr <- substitute(expr)
 
   stopifnot(length(workers) >= 1L, is.character(workers), !anyNA(workers))
@@ -64,7 +64,9 @@ remote <- function(expr, envir=parent.frame(), substitute=TRUE, globals=TRUE, pe
     stop("Argument 'workers' is not of class 'cluster': ", class(workers)[1])
   }
 
-  future <- ClusterFuture(expr=expr, envir=envir, substitute=FALSE, globals=globals, persistent=persistent, workers=workers, user=user, master=myip, revtunnel=revtunnel, homogeneous=homogeneous, gc=gc, earlySignal=earlySignal, label=label, ...)
-  run(future)
+  future <- ClusterFuture(expr=expr, envir=envir, substitute=FALSE, lazy=lazy, seed=seed, globals=globals, persistent=persistent, workers=workers, user=user, master=myip, revtunnel=revtunnel, homogeneous=homogeneous, gc=gc, earlySignal=earlySignal, label=label, ...)
+  if (!future$lazy) future <- run(future)
+  invisible(future)
 }
 class(remote) <- c("remote", "multiprocess", "future", "function")
+attr(remote, "init") <- TRUE
