@@ -457,7 +457,7 @@ objectSize <- function(x, depth = 3L) {
   if (depth <= 0) return(0)
   
   if (!is.list(x) && !is.environment(x)) {
-    size <- object.size(x)
+    size <- unclass(object.size(x))
     x <- environment(x)
   } else {
     size <- 0
@@ -472,12 +472,22 @@ objectSize <- function(x, depth = 3L) {
     FALSE
   }
 
+  objectSize.FutureGlobals <- function(x, ...) {
+    size <- attr(x, "total_size")
+    if (!is.na(size)) return(size)
+    objectSize.list(x, ...)
+  }
+  
   objectSize.list <- function(x, depth) {
     # Nothing to do?
     if (depth <= 0) return(0)
     depth <- depth - 1L
     size <- 0
-    for (kk in seq_along(x)) {
+
+    ## Use the true length that corresponds to what .subset2() uses
+    nx <- .length(x)
+    
+    for (kk in seq_len(nx)) {
       ## NOTE: Use non-class dispatching subsetting to avoid infinite loop,
       ## e.g. x <- packageVersion("future") gives x[[1]] == x.
       x_kk <- .subset2(x, kk)
@@ -486,7 +496,7 @@ objectSize <- function(x, depth = 3L) {
       } else if (is.environment(x_kk)) {
         if (!scanned(x_kk)) size <- size + objectSize.env(x_kk, depth = depth)
       } else {
-        size <- size + object.size(x_kk)
+        size <- size + unclass(object.size(x_kk))
       }
     }
     size
@@ -532,7 +542,7 @@ objectSize <- function(x, depth = 3L) {
           size <- size + objectSize.env(x_kk, depth = depth)
         }
       } else {
-        size <- size + object.size(x_kk)
+        size <- size + unclass(object.size(x_kk))
       }
     }
   
@@ -547,3 +557,120 @@ objectSize <- function(x, depth = 3L) {
 
   size
 }
+
+
+get_random_seed <- function() {
+  env <- globalenv()
+  env$.Random.seed
+}
+
+set_random_seed <- function(seed) {
+  env <- globalenv()
+  if (is.null(seed)) {
+    rm(list = ".Random.seed", envir = env, inherits = FALSE)
+  } else {
+    env$.Random.seed <- seed
+  }
+}
+
+next_random_seed <- function(seed = get_random_seed()) {
+  sample.int(n = 1L, size = 1L, replace = FALSE)
+  seed_next <- get_random_seed()
+  stopifnot(!any(seed_next != seed))
+  invisible(seed_next)
+}
+
+is_valid_random_seed <- function(seed) {
+  oseed <- get_random_seed()
+  on.exit(set_random_seed(oseed))
+  env <- globalenv()
+  env$.Random.seed <- seed
+  res <- tryCatch({
+    sample.int(n = 1L, size = 1L, replace = FALSE)
+  }, FUN = function(w) w)
+  !inherits(res, "warning")
+}
+
+is_lecyer_cmrg_seed <- function(seed) {
+  is.numeric(seed) && length(seed) == 7L &&
+    all(is.finite(seed)) && seed[1] == 407L
+}
+
+# @importFrom utils capture.output
+as_lecyer_cmrg_seed <- function(seed) {
+  ## Generate a L'Ecuyer-CMRG seed (existing or random)?
+  if (is.logical(seed)) {
+    stopifnot(length(seed) == 1L)
+    if (!is.na(seed) && !seed) {
+      stop("Argument 'seed' must be TRUE if logical: ", seed)
+    }
+
+    oseed <- get_random_seed()
+    
+    ## Already a L'Ecuyer-CMRG seed?  Then use that as is.
+    if (!is.na(seed) && seed) {
+      if (is_lecyer_cmrg_seed(oseed)) return(oseed)
+    }
+    
+    ## Otherwise, generate a random one.
+    on.exit(set_random_seed(oseed), add = TRUE)
+    RNGkind("L'Ecuyer-CMRG")
+    return(get_random_seed())
+  }
+
+  stopifnot(is.numeric(seed), all(is.finite(seed)))
+  seed <- as.integer(seed)
+
+  ## Already a L'Ecuyer-CMRG seed?
+  if (length(seed) == 7L) {
+    if (seed[1] != 407L) {
+      stop("Argument 'seed' must be L'Ecuyer-CMRG RNG seed as returned by parallel::nextRNGStream() or an single integer: ", capture.output(str(seed)))
+    }
+    return(seed)
+  }
+  
+  ## Generate a new L'Ecuyer-CMRG seed?
+  if (length(seed) == 1L) {
+    oseed <- get_random_seed()
+    on.exit(set_random_seed(oseed), add = TRUE)
+    RNGkind("L'Ecuyer-CMRG")
+    set.seed(seed)
+    return(get_random_seed())
+  }
+  
+  stop("Argument 'seed' must be of length 1 or 7 (= 1+6):", capture.output(str(seed)))
+}
+
+
+#' Gets the length of an object without dispatching
+#'
+#' @param x Any R object.
+#'
+#' @return A non-negative integer.
+#'
+#' @details
+#' This function returns \code{length(unclass(x))}, but tries to avoid
+#' calling \code{unclass(x)} unless necessary.
+#' 
+#' @seealso \code{\link{.subset}()} and \code{\link{.subset2}()}.
+#' 
+#' @keywords internal
+#' @rdname private_length
+.length <- function(x) {
+  nx <- length(x)
+  
+  ## Can we trust base::length(x), i.e. is there a risk that there is
+  ## a method that overrides with another definition?
+  classes <- class(x)
+  if (length(classes) == 1L && classes == "list") return(nx)
+
+  ## Identify all length() methods for this object
+  mthds <- sprintf("length.%s", classes)
+  keep <- lapply(mthds, FUN = exists, mode = "function", inherits = TRUE)
+  keep <- unlist(keep, use.names = FALSE)
+
+  ## If found, don't trust them
+  if (any(keep)) nx <- length(unclass(x))
+  
+  nx
+} ## .length()
